@@ -1,21 +1,30 @@
 # coding=utf-8
+# Copyright 2018 Sascha Schirra
 #
-# Copyright 2015 Sascha Schirra
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are met:
 #
-# This file is part of Ropper.
+# 1. Redistributions of source code must retain the above copyright notice, this
+# list of conditions and the following disclaimer.
 #
-# Ropper is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# 2. Redistributions in binary form must reproduce the above copyright notice,
+# this list of conditions and the following disclaimer in the documentation
+# and/or other materials provided with the distribution.
 #
-# Ropper is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# 3. Neither the name of the copyright holder nor the names of its contributors
+# may be used to endorse or promote products derived from this software without
+# specific prior written permission.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" A ND
+# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
+# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
+# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
+# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
+# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 from __future__ import print_function
 from ropper.loaders.loader import Loader, Type
 from ropper.printer.printer import FileDataPrinter
@@ -56,7 +65,7 @@ def safe_cmd(func):
         except KeyboardInterrupt:
             cp.println()
         except:
-            cp.printError('Please report this error on https://github.com/sashs/ropper')            
+            cp.printError('Please report this error on https://github.com/sashs/ropper')
             cp.printError( traceback.format_exc())
     return cmd
 
@@ -94,7 +103,7 @@ class CallbackClass(object):
         self.__console.cprinter.printProgress('analyse gadgets...', progress)
         if progress == 1.0:
             self.__console.cprinter.finishProgress()
-            
+
 
     def __ropchainMessages__(self, message):
         if message.startswith('[*]'):
@@ -113,7 +122,7 @@ class Console(cmd.Cmd):
         self.__options = options
         if not options.isWindows():
             import readline
-            old_delims = readline.get_completer_delims() 
+            old_delims = readline.get_completer_delims()
             old_delims = old_delims.replace('-', '')
             old_delims = old_delims.replace('/', '')
             readline.set_completer_delims(old_delims)
@@ -133,7 +142,7 @@ class Console(cmd.Cmd):
     @property
     def cprinter(self):
         return self.__cprinter
-    
+
     @property
     def currentFileName(self):
         if not self.__currentFileName:
@@ -178,7 +187,7 @@ class Console(cmd.Cmd):
 
         if self.__options.console:
             self.cmdloop()
-        
+
         self.__handleOptions(self.__options)
 
     def __updatePrompt(self):
@@ -189,18 +198,18 @@ class Console(cmd.Cmd):
             self.prompt = cstr('(ropper)> ', Color.RED)
 
     def __loadFile(self, file):
-            
+
         self.__rs.addFile(file, raw=self.__options.raw,
                           arch=self.__options.arch)
         self.__options.arch = None
-        
+
         self.__currentFileName = file
         self.__updatePrompt()
         if self.__options.I is not None:
             self.__rs.setImageBaseFor(file, self.__options.I)
         if not self.__options.no_load and self.__options.console:
             self.__loadGadgets()
-        
+
         #self.__binary.printer = FileDataPrinter.create(self.__binary.type)
 
     def __printGadget(self, gadget, detailed=False):
@@ -225,6 +234,9 @@ class Console(cmd.Cmd):
     def __printError(self, error):
         self.__cprinter.printError(error)
 
+    def __printWarning(self, warning):
+        self.__cprinter.printWarning(warning)
+
     def __printInfo(self, info):
         self.__cprinter.printInfo(cstr(info))
 
@@ -285,7 +297,7 @@ class Console(cmd.Cmd):
 
     def __loadGadgetsForAllFiles(self):
         self.__rs.loadGadgetsFor()
-        
+
     def __loadGadgets(self):
         self.__searchGadgetsFor(self.currentFileName)
 
@@ -335,6 +347,8 @@ class Console(cmd.Cmd):
 
     def __generateChain(self, command):
         split = command.split(' ')
+        badbytes = self.__options.badbytes if self.__options.badbytes is not None else ''
+        value_with_badbytes = None
         try:
             old = self.__rs.options.color
             generator = split[0]
@@ -345,8 +359,11 @@ class Console(cmd.Cmd):
                         raise RopperError('Wrong option format. An option has to be set in the following format: option=value')
                     key, value = option.split('=')
                     options[key] = value
+                    badbytes_value = value.encode('ASCII')
+                    if any(badbyte in badbytes_value for badbyte in unhexlify(badbytes)):
+                        value_with_badbytes = value
             try:
-                
+
                 self.__rs.options.color = False
                 chain = self.__rs.createRopChain(generator, str(self.currentFile.arch) ,options)
 
@@ -357,6 +374,8 @@ class Console(cmd.Cmd):
 
                 self.__cprinter.println(chain)
                 # self.__printSeparator(before='\n\n')
+                if value_with_badbytes is not None:
+                    self.__printWarning("The input value {} contains badbytes!".format(value_with_badbytes))
                 self.__printInfo('rop chain generated!')
             except RopperError as e:
                 self.__rs.options.color = old
@@ -405,15 +424,75 @@ class Console(cmd.Cmd):
         else:
             self.__printInfo('No bytes to print')
 
+    def __printExamples(self):
+        self.__cprinter.println("""example uses:
+  [Generic]
+  {0}
+  {0} --file /bin/ls --console
+
+  [Information]
+  {0} --file /bin/ls --info
+  {0} --file /bin/ls --imports
+  {0} --file /bin/ls --sections
+  {0} --file /bin/ls --segments
+  {0} --file /bin/ls --set nx
+  {0} --file /bin/ls --unset nx
+  {0} --file /bin/ls --inst-count 5
+  {0} --file /bin/ls --search "sub eax" --badbytes 000a0d
+  {0} --file /bin/ls --search "sub eax" --detail
+  {0} --file /bin/ls --filter "sub eax"
+  {0} --file /bin/ls --opcode ffe4
+  {0} --file /bin/ls --opcode ffe?
+  {0} --file /bin/ls --opcode ??e4
+  {0} --file /bin/ls --detailed
+  {0} --file /bin/ls --ppr --nocolor
+  {0} --file /bin/ls --jmp esp,eax
+  {0} --file /bin/ls --type jop
+  {0} --file /bin/ls --chain execve
+  {0} --file /bin/ls --chain "execve cmd=/bin/sh" --badbytes 000a0d
+  {0} --file /bin/ls --chain "mprotect address=0xbfdff000 size=0x21000"
+  {0} --file /bin/ls /lib/libc.so.6 --console
+
+
+  [Assemble/Disassemble]
+  {0} --asm "jmp esp"
+  {0} --asm "mov eax, ecx; ret"
+  {0} --disasm ffe4
+
+  [Search]
+  {0} --file /bin/ls --search <searchstring>
+  ?\t\tany character
+  %\t\tany string
+
+  Example:
+
+  {0} --file /bin/ls --search "mov e?x"
+  0x000067f1: mov edx, dword ptr [ebp + 0x14]; mov dword ptr [esp], edx; call eax
+  0x00006d03: mov eax, esi; pop ebx; pop esi; pop edi; pop ebp; ret ;
+  0x00006d6f: mov ebx, esi; mov esi, dword ptr [esp + 0x18]; add esp, 0x1c; ret ;
+  0x000076f8: mov eax, dword ptr [eax]; mov byte ptr [eax + edx], 0; add esp, 0x18; pop ebx; ret ;
+
+  {0} --file /bin/ls --search "mov [%], edx"
+  0x000067ed: mov dword ptr [esp + 4], edx; mov edx, dword ptr [ebp + 0x14]; mov dword ptr [esp], edx; call eax;
+  0x00006f4e: mov dword ptr [ecx + 0x14], edx; add esp, 0x2c; pop ebx; pop esi; pop edi; pop ebp; ret ;
+  0x000084b8: mov dword ptr [eax], edx; ret ;
+  0x00008d9b: mov dword ptr [eax], edx; add esp, 0x18; pop ebx; ret ;
+
+  {0} --file /bin/ls --search "mov [%], edx" --quality 1
+  0x000084b8: mov dword ptr [eax], edx; ret ;
+  \n""".format(sys.argv[0]))
+
     @safe_cmd
     def __handleOptions(self, options):
         if options.sections:
             self.__printData('sections')
+        elif options.help_examples:
+            self.__printExamples();
         elif options.analyse:
-            self.__loadGadgets()
+            self.__loadGadgetsForAllFiles()
             #self.do_analyse(options.analyse)
         elif options.semantic:
-            self.__loadGadgets()
+            self.__loadGadgetsForAllFiles()
             self.do_semantic(options.semantic)
         elif options.symbols:
             self.__printData('symbols')
@@ -489,7 +568,7 @@ class Console(cmd.Cmd):
             self.__loadGadgetsForAllFiles()
             self.__generateChain(options.chain)
         elif self.__options.file:
-            self.__loadGadgets()
+            self.__loadGadgetsForAllFiles()
             if options.search:
                 self.__search(options.search, options.quality)
             else:
@@ -695,7 +774,7 @@ nx\t- Clears the NX-Flag (ELF|PE)"""
 
     def help_inst(self):
         self.__printHelpText(
-            'inst <instructions>', 'searchs instructions in executable sections')
+            'inst <instructions>', 'searches instructions in executable sections')
 
     @safe_cmd
     def do_opcode(self, text):
@@ -707,7 +786,7 @@ nx\t- Clears the NX-Flag (ELF|PE)"""
 
     def help_opcode(self):
         self.__printHelpText(
-            'opcode <opcode>', 'searchs opcode in executable sections\nExample:\nopcode ffe4\nopcode ff4?\nopcode ff??\n\nNot allowed:\nopcode ff?4')
+            'opcode <opcode>', 'searches opcode in executable sections\nExample:\nopcode ffe4\nopcode ff4?\nopcode ff??\n\nNot allowed:\nopcode ff?4')
 
     @safe_cmd
     def do_imagebase(self, text):
@@ -746,7 +825,7 @@ nx\t- Clears the NX-Flag (ELF|PE)"""
 
     def help_jmp(self):
         self.__printHelpText(
-            'jmp <reg[,reg...]>', 'searchs jmp reg instructions')
+            'jmp <reg[,reg...]>', 'searches jmp reg instructions')
 
     @safe_cmd
     def do_detailed(self, text):
@@ -776,7 +855,7 @@ nx\t- Clears the NX-Flag (ELF|PE)"""
                         self.__rs.options[splits[0]] = int(splits[1])
                     else:
                         self.__rs.options[splits[0]] = splits[1]
-                        
+
                 else:
                     raise RopperError('Invalid setting')
             except TypeError as e:
@@ -792,7 +871,8 @@ nx\t- Clears the NX-Flag (ELF|PE)"""
                     'type':'The file is scanned for this type of gadgets. (rop, jop, sys, all)',
                     'detailed':'If on the gadgets will be printed with more detailed information',
                     'inst_count':'The max count of instructions in a gadgets',
-                    'count_of_findings':'The max count of findings which will be printed with semantic search (0 = undefined, default: 5'}
+                    'count_of_findings':'The max count of findings which will be printed with semantic search (0 = undefined, default: 5)',
+                    'multiprocessing':'If on multiple processes will be used for gadget scanning (not supported on Windows.)'}
             for key, value in self.__rs.options.items():
                 if isinstance(value, bool):
                     data.append((cstr(key), cstr('on' if value else 'off'), cstr(desc.get(key,''))))
@@ -999,7 +1079,7 @@ nx\t- Clears the NX-Flag (ELF|PE)"""
             self.__rs.analyseGadgets(self.currentFile)
         constraint = None
         constraints = text.split(';')
-        
+
         split = constraints[-1].split(' ')
         stableRegs = []
         for s in split:
@@ -1030,7 +1110,7 @@ nx\t- Clears the NX-Flag (ELF|PE)"""
         self.__cprinter.println()
 
     def help_semantic(self):
-        self.__printHelpText('semantic', 'Searchs gadgets\nsemantic <constraint>[; <constraint>][ !<stable reg>*]\n\nExample:\nsemantc eax==ebx; ecx==1 !edx !esi\n\nValid constraints:\nreg==reg\nreg==number\nreg==[reg]\nreg<+|-|*|/>=<reg|number|[reg]>')
+        self.__printHelpText('semantic', 'Searches gadgets\nsemantic <constraint>[; <constraint>][ !<stable reg>*]\n\nExample:\nsemantc eax==ebx; ecx==1 !edx !esi\n\nValid constraints:\nreg==reg\nreg==number\nreg==[reg]\nreg<+|-|*|/>=<reg|number|[reg]>')
 
     # @safe_cmd
     # def do_analyse(self, text):
@@ -1056,8 +1136,8 @@ nx\t- Clears the NX-Flag (ELF|PE)"""
     #                     if expr == False:
     #                         continue
     #                     solver.add(expr)
-                    
-                        
+
+
     #                 c = None
     #                 c2 = None
     #                 constraint = self.currentFile.arch.searcher._createConstraint(["rsp=rbx"], g.info)
@@ -1073,7 +1153,7 @@ nx\t- Clears the NX-Flag (ELF|PE)"""
     #     else:
     #         self.__printInfo('No such gadget')
 
-        
+
 
     # @safe_cmd
     # def do_edit(self, text):
@@ -1136,6 +1216,9 @@ class ConsolePrinter(object):
     def printError(self, message):
         self.printMessage(cstr('[ERROR]', Color.RED), message)
 
+    def printWarning(self, message):
+        self.printMessage(cstr('[WARNING]', Color.YELLOW), message)
+
     def printInfo(self, message):
         self.printMessage(cstr('[INFO]', Color.GREEN), message)
 
